// This file contains no license whatsoever. It is provided in the public domain as an example
// of how to use the audio library
//
// Bruce Forsberg
// forsberg@adnc.com
// 
//



#include "audioChain.h"
#include "aflibFile.h"
#include "aflibInfo.h"
#include "aflibAudioFile.h"
#include "aflibAudioMixer.h"

#include "fileInfo.h"

#define PLAY_DELTA  4096 

extern int verbose;

audioChain::audioChain()
:
   _process(false),
   _input(NULL),
	_audiomix(NULL),
   _output(NULL),
   _position(0){}

audioChain::~audioChain()
{
	list<aflibAudioFile*>::iterator it;

	for(it=_mixlist.begin();it != _mixlist.end();it++)
		delete (*it);

   delete _input;
   delete _output;
   delete _audiomix;
}

aflibAudioFile*
audioChain::openAudioFile(
	fileInfo& file)
{
	//Try to open a file module if it fails return NULL
	//Only called by this object

   aflibStatus  status;
	aflibAudioFile* audiotemp;

	if( file.format().length() )
	// Create aflibAudioFile object by passing a format string
   	audiotemp = new aflibAudioFile(file.format(), file.name(),
      											(aflibConfig*)&file, &status);
 	else 
	// Create aflibAudioFile object by passing a format type 
   	audiotemp = new aflibAudioFile(AFLIB_AUTO_TYPE, file.name(),
      											(aflibConfig*)&file, &status);

	if(verbose) aflibInfo::printInfo(*audiotemp);

   if (status != AFLIB_SUCCESS)
   {
		aflibInfo::printError(status);
      delete audiotemp;
      cerr << "Can't open file (" << file.format() << ") " 
																	<< file.name() << endl;
      return NULL;
   }
	return audiotemp;
}

void
audioChain::setupChain(
   fileInfo& infile,
	list<fileInfo*>& mixlist,
	fileInfo& outfile)
{
	// Setup audio chain
	// infile->mixfile1->mixfile2...->outfile
	// infile can be audiodevice for recording to outfile
	// outfile can be audiodevice for playing
	// else convert infile to outfile
	// if mixfiles are given then they will be mixed along with infile
	// and output to outfile

	aflibAudioFile* temp_ptr;
	aflibAudio* parent;
	list<fileInfo*>::iterator mix_it;
	

	//Get input aflibAudioFile object
	if( (_input = openAudioFile(infile)) == NULL ) return;
	
	//Get aflibAudioFile objects for each mixfile
	for(mix_it = mixlist.begin();mix_it != mixlist.end();mix_it++){
		temp_ptr = openAudioFile(*(*mix_it));
		// If we got an object add it to _mixlist
		if(temp_ptr){ 
			_mixlist.push_back(temp_ptr);
		}
	}



	// When creating files aflib needs to have a complete config passed

	// If any output format options used on command line use those
	// else copy input format
	if( !outfile.getSamplesPerSecond() )
		outfile.setSamplesPerSecond( infile.getSamplesPerSecond() );
	if( !outfile.getChannels() )
		outfile.setChannels( infile.getChannels() );
	if( outfile.getSampleSize() == AFLIB_SIZE_UNDEFINED)
		outfile.setSampleSize( infile.getSampleSize() );

	// These can't be set on the command line so just copy
	outfile.setDataEndian( infile.getDataEndian() );
	outfile.setDataOrientation( infile.getDataOrientation() );

// Okay we now have a whole bunch of aflibAudioFile objects along
// with a config for the output but no chain setup and no output
// created.

//    Note: I think this along with _output->addParent(*parent) should work
//				it doesen't though
//  	if( (_output = openAudioFile(outfile)) == NULL) return;

//    Create aflibAudioMixer if we have aflibAudioFile objects to mix
	if(_mixlist.size()){ 

		list<aflibAudioFile*>::iterator it;

		int k;

		// First add the _input to the mix
		// id starts at 1
		if(!_audiomix){ 
			_audiomix = new aflibAudioMixer(*_input);
			int input_channels = _input->getInputConfig().getChannels();

		// This isn't right but we want to do something like this:
		// I think the library should handle scaling volumes a bit
		// better.  A new area for me though I haven't totally looked at it.
		// addmix(id,inchannel,outchannel,volscale)
			for(int j=0; j < input_channels;j++){
				for(int i=0; i < outfile.getChannels(); i++)
					_audiomix->addMix(1,i,j,
							100/outfile.getChannels()/(_mixlist.size()+1));
			}
		}

		// Add mixes for all the mixfiles
		for(k=2,it=_mixlist.begin();it != _mixlist.end();it++,k++)
		{
			_audiomix->addParent(*(*it));
			int input_channels = (*it)->getInputConfig().getChannels();
			for(int j=0; j < input_channels;j++){
				for(int i=0; i < outfile.getChannels(); i++)
					_audiomix->addMix(k,i,j,50/input_channels/(_mixlist.size()+1));
			}
		}
		// We created a mix so it's the parent
		parent = _audiomix;

	} else {
		// We don't have any mixfiles so just use the input as parent
		 parent = _input;
	}

	if(verbose) aflibInfo::printInfo(*parent);

// Doesn't work config not set correctly
//	_output->addParent(*parent);

   aflibStatus  status;

	// Open output file bottom of chain

  	temp_ptr = new aflibAudioFile(*parent,outfile.format(), outfile.name(),
      											(aflibConfig*)&outfile, &status);

   if (status != AFLIB_SUCCESS)
   {
		aflibInfo::printError(status);
      delete temp_ptr;
      cerr << "Can't open file (" << outfile.format() << ") " 
																	<< outfile.name() << endl;
      return;
   }

	if(verbose){ 
		cerr << outfile.name() << endl; 
		aflibInfo::printInfo(*temp_ptr);
	}

	_output = temp_ptr;
   _process = true;
   _position = 0;
}


void
audioChain::id(
   fileInfo&  file)
{
 
   if( (_input = openAudioFile(file)) == NULL) return; 
	
	aflibInfo::printInfo(*_input);

}

void
audioChain::all_formats()
{
   list<string> formats, desc;
//	list< list<string> > extensions;
   list<string>::iterator  it, it2;// it_ext;
	//list< list<string> >::iterator it3;

	// This function returns a list of formats, possible file extensions, and
	// description of the format
	// You can use the extensions to open a file based upon extension 
	// which aflib doesn't do (some files can not be auto detected so
	// at times this is good thing) or use them to filter files in a
	// file dialog.
	// Also you can use the formats list for a dropdown list in a gui app.	

//   aflibFile::returnSupportedFormats(formats, extensions, desc);
   aflibFile::returnSupportedFormats(formats, desc);

	cout << "Format\t\tDescription" << endl;
   for (
		it = formats.begin(), it2 = desc.begin();// it3 = extensions.begin(); 
		it != formats.end(); 
		it++, it2++)//, it3++)
   {
      cout 	<< (*it);
		if((*it).length() < 8) cout << "\t " ; 
		cout << " " << (*it2) << endl;
/* 
				<< " Extensions: ";
			for(it_ext = (*it3).begin(); it_ext != (*it3).end(); it_ext++)
				cout << (*it_ext) << " " ;
		cout << endl;
*/
   }
}


bool
audioChain::process()
{
	// This function continues to process PLAY_DELTA until
	// AFLIB_END_OF_FILE is reached or another error is
	// encountered.
	// If mixfiles are given this routine exits after the 
	// shortest file is mixed with all the others
	// else it exits after the end of the input file

   aflibStatus  status;
   int num_samples = PLAY_DELTA;


   // IF audio chain is not setup correctly then we are done
   if (_output == NULL)
      return (false);

   _output->process(status, _position, num_samples);
   if (status != AFLIB_SUCCESS)
   {
      _process = false;
		aflibInfo::printError(status);
   }
   _position += num_samples;                                                               

   return(_process);
}
