// This file contains no license whatsoever. It is provided in the public domain as an example
// of how to use the audio library
//
// Bruce Forsberg
// forsberg@adnc.com
// 
//


// parseArgs - class used to parse arguments for this application
//
//  This class will parse arguments passed into it in order to process
//  an audio file. There are many kinds of arguments accepted. 
//
//	 osalp <filename> 		will simply autodetect <filename> and play it
//									to /dev/audio
//  If the file cannot be autodected try
//  osalp -a					get a list of availible formats
//
//  Then specify a format
//  osalp -f FORMAT <filename>
//
//  osalp -i <filename> prints information about a file.
//
//  To convert or record try
//
//  osalp <infile> -f FORMAT <outfile>
//
//  To record set <infile> to /dev/audio (or whatever device you want to
//  record from).
//
//  To mix files prepend any filename you want to mix with mix i.e.
//
//  osalp file1 mix file2 mix file3 mix file4 -f DEVICE /dev/dsp4
//
//  will mix file1,file2,file3 and file4 and output the mix to /dev/dsp4
//	 osalp will stop mixing after the end of the shortest file
//
//  before each filename the following args are accepted
//
//  -r					Sampling Rate of file
//	 -c					Number of channels
//  -b					Data is signed byte
//  -B					Data is unsigned byte
//  -w					Data is signed word
//  -W					Data is unsigned word
//


#include "parseArgs.h"
#include "fileInfo.h"
#include "iostream.h"
#include "unistd.h"

int verbose = 0;

parseArgs::parseArgs(
      int argc,
      char * argv[]):_mode(parseArgs::PARSE_PLAY_MODE),_infile(NULL),_outfile(NULL)
{
	doopts(argc, argv);
}

parseArgs::~parseArgs(){
	_mixlist.clear();
	if(_infile) delete _infile;
	if(_outfile) delete _outfile;
}

int
parseArgs::doopts(
      int argc,
      char * argv[])
{

	fileInfo* mixfile;

// Get Input File

	_infile = new fileInfo();
	
	if( fileArgs(_infile, argc,argv) ) return 0;

	if(optind >= argc){
		cerr << "No input file!" << endl;
		_mode = parseArgs::PARSE_HELP_MODE;
		return 0;
	}

	_infile->name( argv[optind] );
	optind++;
	
// See if we have any files to mix

	if( optind < argc ){ 
	while( (!strncmp(argv[optind],"mix",3)) && optind < argc ){
			mixfile = new fileInfo();
			if( fileArgs(mixfile,argc,argv) ) return 0;
			if(optind >= argc){
				cerr << "No filename given!" << endl;
				_mode = parseArgs::PARSE_HELP_MODE;
				return 0;
			}
			optind++;
			mixfile->name( argv[optind] );
			_mixlist.push_back(mixfile);
			optind++;		
			if( optind >= argc ) break;
	}
	}

// Get output file

	_outfile = new fileInfo();
	if( fileArgs(_outfile, argc,argv) ) return 0;

	if(optind >= argc){
		_outfile->name( "/dev/audio" );
		_outfile->format( "DEVICE" );
		return 0;
	}

	_outfile->name( argv[optind] );

	if(_outfile->format().length() == 0 ){
		cerr 	<< "Output type not given for: " << _outfile->name() 
				<< " assuming ( DEVICE )" << endl;
		_outfile->format( "DEVICE" );
	}

	return 0;
		
}

int
parseArgs::fileArgs(
		fileInfo* file,
      int argc,
      char * argv[])
{
	int optchar;
   while (-1 != (optchar = getopt(argc, argv, "+vhaif:r:c:bBwW"))){
		switch(optchar){
			case 'v':
				verbose = 1;
			break;
			case 'h':
         	_mode = parseArgs::PARSE_HELP_MODE;
				return -1;
			break;
			case 'a':
         	_mode = parseArgs::PARSE_ALL_MODE;
				return -1;
			break;
			case 'i':
        		_mode = parseArgs::PARSE_ID_MODE;
			break;
			case 'b':
         	file->setSampleSize(AFLIB_DATA_8S);
			break;
			case 'B':
         	file->setSampleSize(AFLIB_DATA_8U);
			break;
			case 'w':
         	file->setSampleSize(AFLIB_DATA_16S);
			break;
			case 'W':
         	file->setSampleSize(AFLIB_DATA_16U);
			break;

			case 'f':
				if(optind > argc){
         		_mode = parseArgs::PARSE_HELP_MODE;
					cerr << "Must specify a file format from " << argv[0] << " -a" << endl;
					return -1;
			  	} else { 
					file->format(optarg);
				}
			break;
			case 'r':
         	if (optind > argc){
         		_mode = parseArgs::PARSE_HELP_MODE;
	            cerr << "Must specify a sample rate with -r option" << endl;
					return -1;
         	} else {
            	file->setSamplesPerSecond(atoi(optarg));
   	      }
			break;
			case 'c':
         	if (optind > argc){
         		_mode = parseArgs::PARSE_HELP_MODE;
	            cerr << "Must specify channels with -c option" << endl;
					return -1;
         	} else {
            	file->setChannels(atoi(optarg));
   	      }
			break;
		}
	}
	return 0;
}

void
parseArgs::help()
{
  	cerr << "osalp [general options] [ [format options] infile ] [ mix [format options] mixfile ] ... [ [format options] outfile ]" << endl << endl;
	cerr << "mix              \t Specify file to mix to first file " << endl;
	cerr << endl << "general options: " << endl;
	cerr << "-a               \t Print availible formats " << endl;
	cerr << "-h               \t Print this help message " << endl;
	cerr << endl << "format options: " << endl;
	cerr << "-c               \t Number of channels" << endl;
	cerr << "-r               \t Sampling Rate" << endl;
	cerr << "-f               \t File format from osalp -a" << endl;
	cerr << "-b               \t Data is signed byte" << endl;
	cerr << "-B               \t Data is unsigned byte" << endl;
	cerr << "-w               \t Data is signed word" << endl;
	cerr << "-W               \t Data is unsigned word" << endl;

}
